/*
 *  linux/amiga/config.c
 *
 *  Copyright (C) 1993 Hamish Macdonald
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file README.legal in the main directory of this archive
 * for more details.
 */

/*
 * Miscellaneous Amiga stuff
 */

#include <stdarg.h>
#include <linux/config.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/interrupt.h>
#include <linux/mktime.h>
#include <linux/kd.h>
#include <linux/tty.h>
#include <linux/console.h>
#include <linux/bootinfo.h>

#include <linux/amigahw.h>
#include <linux/amigaints.h>

#include <asm/system.h>

#if 0
caddr_t CustomBase;
caddr_t CiaABase;
caddr_t CiaBBase;
caddr_t ChipRamBase;
#endif

extern void amiga_sched_init(isrfunc);
extern unsigned long amiga_keyb_init(unsigned long);
extern int amiga_kbdrate (struct kbd_repeat *);
extern void amiga_init_INTS (void);
extern int amiga_add_isr (unsigned long, isrfunc, int, void *);
extern unsigned long amiga_gettimeoffset (void);
extern void a3000_gettod (struct mktime *);
extern void a2000_gettod (struct mktime *);
extern int amiga_hwclk (int, struct hwclk_time *);
extern int amiga_set_clock_mmss (unsigned long);
extern void amiga_check_partition (struct gendisk *hd, unsigned int dev);
extern void amiga_mksound( unsigned int count, unsigned int ticks );
extern void amiga_floppy_init (void);
extern void amiga_reset (void);
extern int amiga_floppy_change (struct buffer_head *bh);
extern unsigned long amiga_hd_init (unsigned long, unsigned long);
extern void amiga_hd_setup (char *str, int *ints);
extern void amiga_waitbut(void);
extern struct consw ami_con;

void config_amiga(void)
{
    mach_sched_init      = amiga_sched_init;
    mach_keyb_init       = amiga_keyb_init;
    mach_kbdrate         = amiga_kbdrate;
    mach_init_INTS       = amiga_init_INTS;
    mach_add_isr         = amiga_add_isr;
    mach_gettimeoffset   = amiga_gettimeoffset;
    if (boot_info.bi_amiga.model == AMI_3000
	|| boot_info.bi_amiga.model == AMI_4000)
	    mach_gettod  = a3000_gettod;
    else
	    mach_gettod  = a2000_gettod;
    mach_hwclk           = amiga_hwclk;
    mach_set_clock_mmss	 = amiga_set_clock_mmss;
    mach_check_partition = amiga_check_partition;
    mach_mksound         = amiga_mksound;
    mach_floppy_init     = amiga_floppy_init;
    mach_floppy_change   = amiga_floppy_change;
#ifdef CONFIG_AMIGA_IDE
    mach_hd_init	 = amiga_hd_init;
    mach_hd_setup	 = amiga_hd_setup;
#endif /* CONFIG_AMIGA_IDE */
    mach_reset		 = amiga_reset;
    waitbut	         = amiga_waitbut;
    conswitchp	         = &ami_con;

#if 0
    /* set up hardware bases */
    CustomBase  = (caddr_t)CUSTOM_PHYSADDR;
    CiaABase    = (caddr_t)CIAA_PHYSADDR;
    CiaBBase    = (caddr_t)CIAB_PHYSADDR;
    ChipRamBase = (caddr_t)CHIP_PHYSADDR;
#endif

    /* clear all DMA bits */
    custom.dmacon = DMAF_ALL;
    /* ensure that the DMA master bit is set */
    custom.dmacon = DMAF_SETCLR | DMAF_MASTER;

    /* initialize chipram allocator */
    amiga_chip_init ();

    /*
     * if it is an A3000, set the magic bit that forces
     * a hard rekick
     */
    if (boot_info.bi_amiga.model == AMI_3000)
	    *(u_char *)0xde0002 |= 0x80;
}

extern long time_finetune;	/* from kernel/sched.c */

void amiga_sched_init (isrfunc timer_routine)
{
    /* number of ticks depends on NTSC or PAL CIA EClock */
    unsigned short ticks =
	boot_info.bi_amiga.eclock == NTSC_ECLOCK ?
	    NTSC_JIFFY_TICKS : PAL_JIFFY_TICKS;

    /* finetune = (ticks - eclock / HZ) / eclock * 1000000 * 2^24 */
    time_finetune =
      (boot_info.bi_amiga.eclock == NTSC_ECLOCK ? -2109136 : 4966620);

    /* CIA interrupts when counter underflows, so adjust ticks by 1 */
    ticks -= 1;

    ciab.cra &= 0xC0;	 /* turn off timer A, continous mode, from Eclk */
    ciab.talo = ticks % 256;
    ciab.tahi = ticks / 256;

    /* install interrupt service routine for CIAB Timer A */
    add_isr (IRQ_AMIGA_CIAB_TA, timer_routine, 0, NULL);
    /* start timer */
    ciab.cra |= 0x01;
}

#define TICK_SIZE 10000

/* This is always executed with interrupts disabled.  */
unsigned long amiga_gettimeoffset (void)
{
    unsigned short hi, lo, hi2;
    unsigned long ticks, offset = 0;

    /* read CIA B timer A current value */
    hi	= ciab.tahi;
    lo	= ciab.talo;
    hi2 = ciab.tahi;

    if (hi != hi2) {
	lo = ciab.talo;
	hi = hi2;
    }

    ticks = hi << 8 | lo;

    if (boot_info.bi_amiga.eclock == NTSC_ECLOCK) {
#if 0 /* XXX */
/* reading the ICR clears all interrupts.  bad idea! */
	if (ticks > NTSC_JIFFY_TICKS - NTSC_JIFFY_TICKS / 100)
	  /* check for pending interrupt */
	  if (ciab.icr & CIA_ICR_TA)
	    offset = 10000;
#endif
	ticks = (NTSC_JIFFY_TICKS-1) - ticks;
	ticks = (10000 * ticks) / NTSC_JIFFY_TICKS;
    } else {
#if 0 /* XXX */
/* reading the ICR clears all interrupts.  bad idea! */
	if (ticks > PAL_JIFFY_TICKS - PAL_JIFFY_TICKS / 100)
	  /* check for pending interrupt */
	  if (ciab.icr & CIA_ICR_TA)
	    offset = 10000;
#endif
	ticks = (PAL_JIFFY_TICKS-1) - ticks;
	ticks = (10000 * ticks) / NTSC_JIFFY_TICKS;
    }

    return ticks + offset;
}

void a3000_gettod (struct mktime *timep)
{
	volatile struct tod3000 *tod = TOD_3000;

	tod->cntrl1 = TOD3000_CNTRL1_HOLD;

	timep->sec  = tod->second1 * 10 + tod->second2;
	timep->min  = tod->minute1 * 10 + tod->minute2;
	timep->hour = tod->hour1   * 10 + tod->hour2;
	timep->day  = tod->day1    * 10 + tod->day2;
	timep->mon  = tod->month1  * 10 + tod->month2 - 1;
	timep->year = tod->year1   * 10 + tod->year2;

	tod->cntrl1 = TOD3000_CNTRL1_FREE;
}

void a2000_gettod (struct mktime *timep)
{
	volatile struct tod2000 *tod = TOD_2000;

	tod->cntrl1 = TOD2000_CNTRL1_HOLD;

	while (tod->cntrl1 & TOD2000_CNTRL1_BUSY)
		;

	timep->sec  = tod->second1     * 10 + tod->second2;
	timep->min  = tod->minute1     * 10 + tod->minute2;
	timep->hour = (tod->hour1 & 3) * 10 + tod->hour2;
	timep->day  = tod->day1        * 10 + tod->day2;
	timep->mon  = tod->month1      * 10 + tod->month2 - 1;
	timep->year = tod->year1       * 10 + tod->year2;

	if (!(tod->cntrl3 & TOD2000_CNTRL3_24HMODE))
		if ((!tod->hour1 & TOD2000_HOUR1_PM) && timep->hour == 12)
			timep->hour = 0;
		else if ((tod->hour1 & TOD2000_HOUR1_PM) && timep->hour != 12)
			timep->hour += 12;

	tod->cntrl1 &= ~TOD2000_CNTRL1_HOLD;
}

int amiga_hwclk(int op, struct hwclk_time *t)
{
	if (boot_info.bi_amiga.model == AMI_3000
	    || boot_info.bi_amiga.model == AMI_4000) {
		volatile struct tod3000 *tod = TOD_3000;

		tod->cntrl1 = TOD3000_CNTRL1_HOLD;

		if (!op) { /* read */
			t->sec  = tod->second1 * 10 + tod->second2;
			t->min  = tod->minute1 * 10 + tod->minute2;
			t->hour = tod->hour1   * 10 + tod->hour2;
			t->day  = tod->day1    * 10 + tod->day2;
			t->wday = tod->weekday;
			t->mon  = tod->month1  * 10 + tod->month2 - 1;
			t->year = tod->year1   * 10 + tod->year2;
		} else {
			tod->second1 = t->sec / 10;
			tod->second2 = t->sec % 10;
			tod->minute1 = t->min / 10;
			tod->minute2 = t->min % 10;
			tod->hour1   = t->hour / 10;
			tod->hour2   = t->hour % 10;
			tod->day1    = t->day / 10;
			tod->day2    = t->day % 10;
			if (t->wday != -1)
				tod->weekday = t->wday;
			tod->month1  = (t->mon + 1) / 10;
			tod->month2  = (t->mon + 1) % 10;
			tod->year1   = t->year / 10;
			tod->year2   = t->year % 10;
		}

		tod->cntrl1 = TOD3000_CNTRL1_FREE;
	} else {
		volatile struct tod2000 *tod = TOD_2000;

		tod->cntrl1 = TOD2000_CNTRL1_HOLD;
	    
		while (tod->cntrl1 & TOD2000_CNTRL1_BUSY)
			;

		if (!op) { /* read */
			t->sec  = tod->second1     * 10 + tod->second2;
			t->min  = tod->minute1     * 10 + tod->minute2;
			t->hour = (tod->hour1 & 3) * 10 + tod->hour2;
			t->day  = tod->day1        * 10 + tod->day2;
			t->wday = tod->weekday;
			t->mon  = tod->month1      * 10 + tod->month2 - 1;
			t->year = tod->year1       * 10 + tod->year2;

			if (!(tod->cntrl3 & TOD2000_CNTRL3_24HMODE))
				if ((!tod->hour1 & TOD2000_HOUR1_PM) && t->hour == 12)
					t->hour = 0;
				else if ((tod->hour1 & TOD2000_HOUR1_PM) && t->hour != 12)
					t->hour += 12;
		} else {
			tod->second1 = t->sec / 10;
			tod->second2 = t->sec % 10;
			tod->minute1 = t->min / 10;
			tod->minute2 = t->min % 10;
			if (tod->cntrl3 & TOD2000_CNTRL3_24HMODE)
				tod->hour1 = t->hour / 10;
			else if (t->hour >= 12)
				tod->hour1 = TOD2000_HOUR1_PM +
					(t->hour - 12) / 10;
			else
				tod->hour1 = t->hour / 10;
			tod->hour2   = t->hour % 10;
			tod->day1    = t->day / 10;
			tod->day2    = t->day % 10;
			if (t->wday != -1)
				tod->weekday = t->wday;
			tod->month1  = (t->mon + 1) / 10;
			tod->month2  = (t->mon + 1) % 10;
			tod->year1   = t->year / 10;
			tod->year2   = t->year % 10;
		}

		tod->cntrl1 &= ~TOD2000_CNTRL1_HOLD;
	}

	return 0;
}

int amiga_set_clock_mmss (unsigned long nowtime)
{
	short real_seconds = nowtime % 60, real_minutes = (nowtime / 60) % 60;

	if (boot_info.bi_amiga.model == AMI_3000
	    || boot_info.bi_amiga.model == AMI_4000) {
		volatile struct tod3000 *tod = TOD_3000;

		tod->cntrl1 = TOD3000_CNTRL1_HOLD;

		tod->second1 = real_seconds / 10;
		tod->second2 = real_seconds % 10;
		tod->minute1 = real_minutes / 10;
		tod->minute2 = real_minutes % 10;
		
		tod->cntrl1 = TOD3000_CNTRL1_FREE;
	} else {
		volatile struct tod2000 *tod = TOD_2000;

		tod->cntrl1 = TOD2000_CNTRL1_HOLD;
	    
		while (tod->cntrl1 & TOD2000_CNTRL1_BUSY)
			;

		tod->second1 = real_seconds / 10;
		tod->second2 = real_seconds % 10;
		tod->minute1 = real_minutes / 10;
		tod->minute2 = real_minutes % 10;

		tod->cntrl1 &= ~TOD2000_CNTRL1_HOLD;
	}

	return 0;
}

void amiga_waitbut (void)
{
    int i;

    while (1) {
	while (ciaa.pra & 0x40);

	/* debounce */
	for (i = 0; i < 1000; i++);

	if (!(ciaa.pra & 0x40))
	    break;
    }

    /* wait for button up */
    while (1) {
	while (!(ciaa.pra & 0x40));

	/* debounce */
	for (i = 0; i < 1000; i++);

	if (ciaa.pra & 0x40)
	    break;
    }
}

void ami_serial_print (const char *str)
{
	ushort c;

	while (*str) {
		c = *str++;

		custom.serdat = c | 0x100;
		while (!(custom.serdatr & 0x2000))
			;
		if (c == '\n') {
			custom.serdat = '\r' | 0x100;
			while (!(custom.serdatr & 0x2000))
				;
		}
	}
}

void dbprintf(const char *fmt , ...)
{
	static char buf[1024];
	va_list args;
	extern void console_print (const char *str);
	extern int vsprintf(char * buf, const char * fmt, va_list args);

	va_start(args, fmt);
	vsprintf(buf, fmt, args);
	va_end(args);

	console_print (buf);
}

NORET_TYPE void amiga_reset( void )
    ATTRIB_NORET;

void amiga_reset (void)
{
  unsigned long jmp_addr040 = VTOP(&&jmp_addr_label040);
  unsigned long jmp_addr = VTOP(&&jmp_addr_label);

  cli();
  if (boot_info.cputype & CPU_68040)
    /* Setup transparent translation registers for mapping
     * of 16 MB kernel segment before disabling translation
     */
    __asm__ __volatile__
      ("movel    %0,d0\n\t"
       "andl     #0xff000000,d0\n\t"
       "orw      #0xe020,d0\n\t"   /* map 16 MB, enable, cacheable */
       ".long    0x4e7b0004\n\t"   /* movec d0,itt0 */
       ".long    0x4e7b0006\n\t"   /* movec d0,dtt0 */
       "jmp      %0@\n\t"
       : /* no outputs */
       : "a" (jmp_addr040));
  else
    /* for 680[23]0, just disable translation and jump to the physical
     * address of the label
     */
    __asm__ __volatile__
      ("pmove  tc,sp@\n\t"
       "bclr   #7,sp@\n\t"
       "pmove  sp@,tc\n\t"
       "jmp    %0@\n\t"
       : /* no outputs */
       : "a" (jmp_addr));
 jmp_addr_label040:
  /* disable translation on '040 now */
  __asm__ __volatile__    
    ("moveq #0,d0\n\t"
     ".long 0x4e7b0003\n\t"         /* movec d0,tc; disable MMU */
     : /* no outputs */
     : /* no inputs */
     : "d0");

 jmp_addr_label:
  /* pickup reset address from AmigaOS ROM, reset devices and jump
   * to reset address
   */
  __asm__ __volatile__
    ("movew #0x2700,sr\n\t"
     "leal  0x01000000,a0\n\t"
     "subl  a0@(-0x14),a0\n\t"
     "movel a0@(4),a0\n\t"
     "subql #2,a0\n\t"
     "bra   1f\n\t"
     /* align on a longword boundary */
     ".align 2\n\t"
     "1:\n\t"
     "reset\n\t"
     "jmp   a0@");
  
  for (;;);

}
