/*
 *  linux/atari/config.c
 *
 *  Copyright (C) 1994 Bjrn Brauel
 *
 *  5/2/94 Roman Hodek:
 *    Added setting of time_adj to get a better clock.
 *
 *  5/14/94 Roman Hodek:
 *    gettod() for TT 
 *
 *  5/15/94 Roman Hodek:
 *    hard_reset_now() for Atari (and others?)
 *
 *  94/12/30 Andreas Schwab:
 *    atari_sched_init fixed to get precise clock.
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file README.legal in the main directory of this archive
 * for more details.
 */

/*
 * Miscellaneous atari stuff
 */

#include <linux/config.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/mktime.h>
#include <linux/mm.h>
#include <linux/bootinfo.h>
#include <linux/mc146818rtc.h>
#include <linux/kd.h>
#include <linux/tty.h>
#include <linux/console.h>

#include <linux/atarihw.h>
#include <linux/atariints.h>

#include <asm/system.h>
#include <asm/io.h>

extern void atari_sched_init(isrfunc);
extern unsigned long atari_keyb_init(unsigned long);
extern int atari_kbdrate (struct kbd_repeat *);
extern void atari_init_INTS (void);
extern int atari_add_isr (unsigned long, isrfunc, int, void *);
extern void atari_process_int (int level, struct intframe *fp);
extern unsigned long atari_gettimeoffset (void);
extern void atari_mste_gettod (struct mktime *);
extern void atari_gettod (struct mktime *);
extern int atari_mste_hwclk (int, struct hwclk_time *);
extern int atari_hwclk (int, struct hwclk_time *);
extern int atari_mste_set_clock_mmss (unsigned long);
extern int atari_set_clock_mmss (unsigned long);
extern void atari_check_partition (struct gendisk *hd, unsigned int dev);
extern void atari_mksound( unsigned int count, unsigned int ticks );
extern void atari_reset( void );
extern void atari_floppy_init( void );
extern int atari_floppy_change( struct buffer_head *bh );
extern unsigned long atari_hd_init (unsigned long, unsigned long);
extern void atari_hd_setup (char *str, int *ints);
extern struct consw ata_con;


/* This function tests for the presence of an address, specially a
 * hardware register address. It is called very early in the kernel
 * initialization process, when the VBR register isn't set up yet. On
 * an Atari, it still points to address 0, which is unmapped. So a bus
 * error would cause another bus error while fetching the exception
 * vector, and the CPU would do nothing at all. So we needed to set up
 * a temporary VBR and a vector table for the duration of the test.
 */

static int hwreg_present( volatile void *regp )

{	int		ret;
	long	save_sp, save_vbr;
	long	tmp_vectors[3];
	
	__asm__ __volatile__
	(	"movec	vbr,%2\n\t"
		"movel	#Lberr1,%4@(8)\n\t"
	    "movec	%4,vbr\n\t"
		"movel	sp,%1\n\t"
		"moveq	#0,%0\n\t"
		"tstb	%3@\n\t"  
		"moveq	#1,%0\n"
	"Lberr1:\n\t"
		"movel	%1,sp\n\t"
		"movec	%2,vbr"
		: "=&d" (ret), "=&r" (save_sp), "=&r" (save_vbr)
		: "a" (regp), "a" (tmp_vectors)
	);

	return( ret );
}

extern int is_medusa;

void config_atari(void)
{
	mach_sched_init      = atari_sched_init;
	mach_keyb_init       = atari_keyb_init;
	mach_kbdrate         = atari_kbdrate;
	mach_init_INTS       = atari_init_INTS;
	mach_add_isr         = atari_add_isr;
	mach_process_int	 = atari_process_int;
	mach_gettimeoffset   = atari_gettimeoffset;
	mach_check_partition = atari_check_partition;
	mach_mksound         = atari_mksound;
	mach_reset           = atari_reset;
	mach_floppy_init	 = atari_floppy_init;
	mach_floppy_change   = atari_floppy_change;
#ifdef CONFIG_ATARI_FALCON_IDE
	mach_hd_init	 = atari_hd_init;
	mach_hd_setup	 = atari_hd_setup;
#endif
	conswitchp	         = &ata_con;

	/* ++bjoern: 
	 * Determine hardware present
	 */

	printk( "Atari hardware found: " );
	if (is_medusa) {
		/* There's no Atari video hardware on the Medusa, but all the
		 * addresses below generate a DTACK so no bus error occurs! */
	}
	else if (hwreg_present( f030_xreg )) {
		boot_info.bi_atari.hw_present |= ATARIHW_VIDEL_SHIFTER;
		printk( "VIDEL " );
		/* This is a temporary hack: If there is Falcon video
		 * hardware, we assume that the STDMA servers SCSI instead of
		 * ACSI. In the future, there should be a better method for
		 * this...
		 */
		boot_info.bi_atari.hw_present |= ATARIHW_STND_SCSI;
		printk( "STDMA-SCSI " );
	}
	else if (hwreg_present( tt_palette )) {
		boot_info.bi_atari.hw_present |= ATARIHW_TT_SHIFTER;
		printk( "TT_SHIFTER " );
	}
	else if (hwreg_present( &shifter.bas_hi )) {
		if (hwreg_present( &shifter.bas_lo )) {
			boot_info.bi_atari.hw_present |= ATARIHW_EXTD_SHIFTER;
			printk( "EXTD_SHIFTER " );
		}
		else {
			boot_info.bi_atari.hw_present |= ATARIHW_STND_SHIFTER;
			printk( "STND_SHIFTER " );
		}
	}
	if (hwreg_present( &mfp.par_dt_reg )) {
		boot_info.bi_atari.hw_present |= ATARIHW_ST_MFP;
		printk( "ST_MFP " );
	}
	if (hwreg_present( &tt_mfp.par_dt_reg )) {
		boot_info.bi_atari.hw_present |= ATARIHW_TT_MFP;
		printk( "TT_MFP " );
	}
	if (hwreg_present( &tt_scsi_dma.dma_addr_hi )) {
		boot_info.bi_atari.hw_present |= ATARIHW_SCSI_DMA;
		printk( "TT_SCSI_DMA " );
	}
	if (hwreg_present( &st_dma.dma_hi )) {
		boot_info.bi_atari.hw_present |= ATARIHW_STND_DMA;
		printk( "STND_DMA " );
	}
	if (is_medusa || /* The ST-DMA address registers aren't readable
					  * on all Medusas, so the test below may fail */
		(hwreg_present( &st_dma.dma_vhi ) &&
		 (st_dma.dma_vhi = 0x55) && (st_dma.dma_hi = 0xaa) &&
		 st_dma.dma_vhi == 0x55 && st_dma.dma_hi == 0xaa &&
		 (st_dma.dma_vhi = 0xaa) && (st_dma.dma_hi = 0x55) &&
		 st_dma.dma_vhi == 0xaa && st_dma.dma_hi == 0x55)) {
		boot_info.bi_atari.hw_present |= ATARIHW_EXTD_DMA;
		printk( "EXTD_DMA " );
	}
	if (hwreg_present( &tt_scsi.scsi_data )) {
		boot_info.bi_atari.hw_present |= ATARIHW_TT_SCSI;
		printk( "TT_SCSI " );
	}
	if (hwreg_present( &sound_ym.rd_data_reg_sel )) {
		boot_info.bi_atari.hw_present |= ATARIHW_YM_2149;
		printk( "YM2149 " );
	}
	if (!is_medusa && hwreg_present( &tt_dmasnd.ctrl )) {
		boot_info.bi_atari.hw_present |= ATARIHW_PCM_8BIT;
		printk( "PCM " );
	}
#if 0 /* This is WRONG! */
	if (hwreg_present( (void *)(0xffff8900) )) {
		boot_info.bi_atari.hw_present |= ATARIHW_CODEC_16BIT;
		printk( "CODEC " );
	}
#endif
	if (!is_medusa && hwreg_present( &tt_scc_dma.dma_addr_hi )) {
		boot_info.bi_atari.hw_present |= ATARIHW_SCC_DMA;
		printk( "SCC_DMA " );
	}
	if (!is_medusa && hwreg_present( &scc.cha_a_ctrl )) {
		boot_info.bi_atari.hw_present |= ATARIHW_SCC;
		printk( "SCC " );
	}
	if (hwreg_present( &tt_scu.sys_mask )) {
		boot_info.bi_atari.hw_present |= ATARIHW_VME;
		boot_info.bi_atari.hw_present |= ATARIHW_SCU;
		printk( "VME SCU " );
	}
	if (hwreg_present( (void *)(0xffff9210) )) {
		boot_info.bi_atari.hw_present |= ATARIHW_ANALOG_JOY;
		printk( "ANALOG_JOY " );
	}
	if (hwreg_present( blitter.halftone )) {
		boot_info.bi_atari.hw_present |= ATARIHW_BLITTER;
		printk( "BLITTER " );
	}
	if (hwreg_present( (void *)(0xfff00000) )) {
		boot_info.bi_atari.hw_present |= ATARIHW_IDE;
		printk( "IDE " );
	}
	if (!is_medusa && hwreg_present( &tt_microwire.data )) {
		boot_info.bi_atari.hw_present |= ATARIHW_MCW;
		printk( "MICROWIRE " );
	}
	if (hwreg_present( &tt_rtc.regsel )) {
		boot_info.bi_atari.hw_present |= ATARIHW_TT_CLK;
		printk( "TT_CLK " );
		mach_gettod = atari_gettod;
		mach_hwclk = atari_hwclk;
		mach_set_clock_mmss = atari_set_clock_mmss;
	}
	if (hwreg_present( &mste_rtc.sec_ones)) {
		boot_info.bi_atari.hw_present |= ATARIHW_MSTE_CLK;
		printk( "MSTE_CLK ");
		mach_gettod = atari_mste_gettod;
		mach_hwclk = atari_mste_hwclk;
		mach_set_clock_mmss = atari_mste_set_clock_mmss;
	}
	if (!(boot_info.bi_atari.hw_present & ATARIHW_STND_SCSI)) {
		boot_info.bi_atari.hw_present |= ATARIHW_STND_ACSI;
		printk( "ACSI " );
	}
	printk("\n");

	if (boot_info.cputype & CPU_68040)
		/* Now it seems to be safe to turn of the tt0 transparent
		 * translation (the one that must not be turned off in
		 * head.S...)
		 */
		__asm__ volatile ("moveq #0,d0;"
						  ".long 0x4e7b0004;"	/* movec d0,itt0 */
						  ".long 0x4e7b0006;"	/* movec d0,dtt0 */
						  : /* no outputs */
						  : /* no inputs */
						  : "d0");
	
	/* allocator for memory that must reside in st-ram */
	atari_stram_init ();

	/* Set up a mapping for the VMEbus address region:
	 *
	 * VME is either at phys. 0xfexxxxxx (TT) or 0xa00000..0xdfffff
	 * (MegaSTE) In both cases, the whole 16 MB chunk is mapped at
	 * 0xfe000000 virt., because this can be done with a single
	 * transparent translation. On the 68040, lots of often unused
	 * page tables would be needed otherwise. On a MegaSTE or similar,
	 * the highest byte is stripped off by hardware due to the 24 bit
	 * design of the bus.
	 */

	if (boot_info.cputype & CPU_68030) {
		unsigned long	tt1_val;
		tt1_val = 0xfe008543;	/* Translate 0xfexxxxxx, enable, cache
								 * inhibit, read and write, FDC mask = 3,
								 * FDC val = 4 -> Supervisor only */
		__asm__ __volatile__ ( "pmove	%0@,tt1" : : "a" (&tt1_val) );
	}
	else if (boot_info.cputype & CPU_68040) {
		__asm__ __volatile__
			( "movel %0,d0\n\t"
			  ".long 0x4e7b0005\n\t"	/* movec d0,itt1 */
			  ".long 0x4e7b0007"		/* movec d0,dtt1 */
			  :
			  : "g" (0xfe00a040)		/* Translate 0xfexxxxxx, enable,
										 * supervisor only, non-cacheable/
										 * serialized, writable */
			  : "d0" );
							   
	}
}

extern long time_finetune;	/* from kernel/sched.c */

void atari_sched_init (isrfunc timer_routine)
{
  /* set Timer C data Register */
  mfp.tim_dt_c = INT_TICKS;

  /* install interrupt service routine for MFP Timer C */
  add_isr (IRQ_MFP_TIMC, timer_routine, 0, NULL);
  /* start timer C, div = 1:100 */
  mfp.tim_ct_cd = (mfp.tim_ct_cd & 15) | 0x60; 
  /* enable timer C Interrupt */
  mfp.int_en_b |= 0x20;

  /* The exact frequency of our timer is 2.4576 MHz / 100 / 246 =
     99.9024390244 Hz.  Set time_finetune to correct this.  It is the
     number of micro seconds per jiffies (scaled 2^24) to add to
     kernel real time.  */
  time_finetune = 163840000;
}

/* ++andreas: gettimeoffset fixed to check for pending interrupt */

#define TICK_SIZE 10000

/* This is always executed with interrupts disabled.  */
unsigned long atari_gettimeoffset (void)
{
  unsigned long ticks, offset = 0;

  /* read MFP timer C current value */
  ticks = mfp.tim_dt_c;
  /* The probability of underflow is less than 2% */
  if (ticks > INT_TICKS - INT_TICKS / 50)
    /* Check for pending timer interrupt */
    if (mfp.int_pn_b & (1 << 5))
      offset = TICK_SIZE;

  ticks = INT_TICKS - ticks;
  ticks = ticks * 10000L / INT_TICKS;

  return ticks + offset;
}


static void
mste_read(struct MSTE_RTC *val)
{
#define COPY(v) val->v=(mste_rtc.v & 0xf)
	do {
		COPY(sec_ones) ; COPY(sec_tens) ; COPY(min_ones) ; 
		COPY(min_tens) ; COPY(hr_ones) ; COPY(hr_tens) ; 
		COPY(weekday) ; COPY(day_ones) ; COPY(day_tens) ; 
		COPY(mon_ones) ; COPY(mon_tens) ; COPY(year_ones) ;
		COPY(year_tens) ;
	/* prevent from reading the clock while it changed */
	} while (val->sec_ones != (mste_rtc.sec_ones & 0xf));
#undef COPY
}

static void
mste_write(struct MSTE_RTC *val)
{
#define COPY(v) mste_rtc.v=val->v
	do {
		COPY(sec_ones) ; COPY(sec_tens) ; COPY(min_ones) ; 
		COPY(min_tens) ; COPY(hr_ones) ; COPY(hr_tens) ; 
		COPY(weekday) ; COPY(day_ones) ; COPY(day_tens) ; 
		COPY(mon_ones) ; COPY(mon_tens) ; COPY(year_ones) ;
		COPY(year_tens) ;
	/* prevent from writing the clock while it changed */
	} while (val->sec_ones != (mste_rtc.sec_ones & 0xf));
#undef COPY
}

#define	RTC_READ(reg)							\
    ({	unsigned char	__val;					\
		outb(reg,&tt_rtc.regsel);				\
		__val = tt_rtc.data;					\
		__val;									\
	})

#define	RTC_WRITE(reg,val)						\
    do {										\
		outb(reg,&tt_rtc.regsel);				\
		tt_rtc.data = (val);					\
	} while(0)
    

void atari_mste_gettod( struct mktime *time )
{
	int hr24=0;
	struct MSTE_RTC val;

	mste_rtc.mode=(mste_rtc.mode | 1);
	hr24=mste_rtc.mon_tens & 1;
	mste_rtc.mode=(mste_rtc.mode & ~1);

	mste_read(&val);
	time->sec = val.sec_ones + val.sec_tens * 10;
	time->min = val.min_ones + val.min_tens * 10;
	if (hr24)
		time->hour = val.hr_ones + val.hr_tens * 10;
	else {
		time->hour = val.hr_ones + (val.hr_tens & 1) * 10;
		if (val.hr_tens & 2)
			time->hour += 12;
	}
	time->day = val.day_ones + val.day_tens * 10;
	time->mon = val.mon_ones + val.mon_tens * 10 - 1;
	time->year = val.year_ones + val.year_tens * 10 + 80;	
}


void atari_gettod( struct mktime *time )

{
		unsigned char	ctrl;
		unsigned short 	tos_version;
		
		while (!(RTC_READ(RTC_FREQ_SELECT) & RTC_UIP)) ;
		while (RTC_READ(RTC_FREQ_SELECT) & RTC_UIP) ;

		time->sec  = RTC_READ(RTC_SECONDS);
		time->min  = RTC_READ(RTC_MINUTES);
		time->hour = RTC_READ(RTC_HOURS);
		time->day  = RTC_READ(RTC_DAY_OF_MONTH);
		time->mon  = RTC_READ(RTC_MONTH);
		time->year = RTC_READ(RTC_YEAR);

		ctrl = RTC_READ(RTC_CONTROL); 

		if (!(ctrl & RTC_DM_BINARY)) {
			BCD_TO_BIN(time->sec);
			BCD_TO_BIN(time->min);
			BCD_TO_BIN(time->hour);
			BCD_TO_BIN(time->day);
			BCD_TO_BIN(time->mon);
			BCD_TO_BIN(time->year);
		}
		if (!(ctrl & RTC_24H)) {
			if (time->hour & 0x80) {
				time->hour &= ~0x80;
				time->hour += 12;
			}
		}
	/* Adjust values (let the setup valid) */
	time->mon--;
	/* Tos version at Physical 2 */
	tos_version = *(unsigned short *)PTOV( 2 );
	time->year += (tos_version < 0x306) ? 70 : 68;
}


int atari_mste_hwclk( int op, struct hwclk_time *t )
{	int hour, year;
	int hr24=0;
	struct MSTE_RTC val;

  	mste_rtc.mode=(mste_rtc.mode | 1);
  	hr24=mste_rtc.mon_tens & 1;
  	mste_rtc.mode=(mste_rtc.mode & ~1);

	if (op) {
		/* write: prepare values */

		val.sec_ones = t->sec % 10;
		val.sec_tens = t->sec / 10;
		val.min_ones = t->min % 10;
		val.min_tens = t->min / 10;
		hour = t->hour;
		val.hr_ones = hour % 10;
		val.hr_tens = hour / 10;
		if (!hr24  && hour > 11) {
			hour -= 12;
			val.hr_ones = hour % 10;
			val.hr_tens = (hour / 10) | 2;
		}
		val.day_ones = t->day % 10;
		val.day_tens = t->day / 10;
		val.mon_ones = (t->mon+1) % 10;
		val.mon_tens = (t->mon+1) / 10;
		year = t->year - 80;
		val.year_ones = year % 10;
		val.year_tens = year / 10;
		val.weekday = t->wday;
		mste_write(&val);
	  	mste_rtc.mode=(mste_rtc.mode | 1);
		val.year_ones = (year % 4);	/* leap year register */
	  	mste_rtc.mode=(mste_rtc.mode & ~1);
	}
	else {
		mste_read(&val);
		t->sec = val.sec_ones + val.sec_tens * 10;
		t->min = val.min_ones + val.min_tens * 10;
		if (hr24)
			t->hour = val.hr_ones + val.hr_tens * 10;
		else {
			t->hour = val.hr_ones + (val.hr_tens & 1) * 10;
			if (val.hr_tens & 2)
				t->hour += 12;
		}
		t->mon = val.mon_ones + val.mon_tens * 10 - 1;
		t->year = val.year_ones + val.year_tens * 10 + 80;
		t->wday = val.weekday;
	}
	return 0;
}

#define HWCLK_POLL_INTERVAL	5

int atari_hwclk( int op, struct hwclk_time *t )

{	int sec=0, min=0, hour=0, day=0, mon=0, year=0, wday=0; 
	unsigned long 	flags;
	unsigned short	tos_version;
	unsigned char	ctrl;
	
	/* Tos version at Physical 2 */
	tos_version = *(unsigned short *)PTOV( 2 );

	ctrl = RTC_READ(RTC_CONTROL); /* control registers are
				       * independant from the UIP */

	if (op) {
		/* write: prepare values */

		sec  = t->sec;
		min  = t->min;
		hour = t->hour;
		day  = t->day;
		mon  = t->mon + 1;
		year = t->year - ((tos_version < 0x306) ? 70 : 68);
		wday = t->wday + (t->wday >= 0);
		
		if (!(ctrl & RTC_24H) && hour > 11) {
			hour -= 12;
			hour |= 0x80;
		}

		if (!(ctrl & RTC_DM_BINARY)) {
			BIN_TO_BCD(sec);
			BIN_TO_BCD(min);
			BIN_TO_BCD(hour);
			BIN_TO_BCD(day);
			BIN_TO_BCD(mon);
			BIN_TO_BCD(year);
			if (wday >= 0) BIN_TO_BCD(wday);
		}
	}

	/* Reading/writing the clock registers is a bit critical due to
	 * the regular update cycle of the RTC. While an update is in
	 * progress, registers 0..9 shouldn't be touched.
	 * The problem is solved like that: If an update is currently in
	 * progress (the UIP bit is set), the process sleeps for a while
	 * (50ms). This really should be enough, since the update cycle
	 * normally needs 2 ms.
	 * If the UIP bit reads as 0, we have at least 244 usecs until the
	 * update starts. This should be enough... But to be sure,
	 * additionally the RTC_SET bit is set to prevent an update cycle.
	 */

	while( RTC_READ(RTC_FREQ_SELECT) & RTC_UIP ) {
		current->state = TASK_INTERRUPTIBLE;
		current->timeout = jiffies + HWCLK_POLL_INTERVAL;
		schedule();
	}

	save_flags(flags);
	cli();
	RTC_WRITE( RTC_CONTROL, ctrl | RTC_SET );
	if (!op) {
		sec  = RTC_READ( RTC_SECONDS );
		min  = RTC_READ( RTC_MINUTES );
		hour = RTC_READ( RTC_HOURS );
		day  = RTC_READ( RTC_DAY_OF_MONTH );
		mon  = RTC_READ( RTC_MONTH );
		year = RTC_READ( RTC_YEAR );
		wday = RTC_READ( RTC_DAY_OF_WEEK );
	}
	else {
		RTC_WRITE( RTC_SECONDS, sec );
		RTC_WRITE( RTC_MINUTES, min );
		RTC_WRITE( RTC_HOURS, hour );
		RTC_WRITE( RTC_DAY_OF_MONTH, day );
		RTC_WRITE( RTC_MONTH, mon );
		RTC_WRITE( RTC_YEAR, year );
		if (wday >= 0) RTC_WRITE( RTC_DAY_OF_WEEK, wday );
	}
	RTC_WRITE( RTC_CONTROL, ctrl & ~RTC_SET );
	restore_flags(flags);

	if (!op) {
		/* read: adjust values */

		if (!(ctrl & RTC_DM_BINARY)) {
			BCD_TO_BIN(sec);
			BCD_TO_BIN(min);
			BCD_TO_BIN(hour);
			BCD_TO_BIN(day);
			BCD_TO_BIN(mon);
			BCD_TO_BIN(year);
			BCD_TO_BIN(wday);
		}

		if (!(ctrl & RTC_24H)) {
			if (hour & 0x80) {
				hour &= ~0x80;
				hour += 12;
			}
		}

		t->sec  = sec;
		t->min  = min;
		t->hour = hour;
		t->day  = day;
		t->mon  = mon - 1;
		t->year = year + ((tos_version < 0x306) ? 70 : 68);
		t->wday = wday - 1;
	}

	return( 0 );
}


int atari_mste_set_clock_mmss (unsigned long nowtime)
{
  short real_seconds = nowtime % 60, real_minutes = (nowtime / 60) % 60;
  struct MSTE_RTC val;
  unsigned char rtc_minutes;

  mste_read(&val);  
  rtc_minutes= val.min_ones + val.min_tens * 10;
  if ((rtc_minutes < real_minutes
       ? real_minutes - rtc_minutes
       : rtc_minutes - real_minutes) < 30)
    {
      val.sec_ones = real_seconds % 10;
      val.sec_tens = real_seconds / 10;
      val.min_ones = real_minutes % 10;
      val.min_tens = real_minutes / 10;
      mste_write(&val);
    }
  else
    return -1;
  return 0;
}

int atari_set_clock_mmss (unsigned long nowtime)
{
  int retval = 0;
  short real_seconds = nowtime % 60, real_minutes = (nowtime / 60) % 60;
  unsigned char save_control, save_freq_select, rtc_minutes;

  save_control = RTC_READ (RTC_CONTROL); /* tell the clock it's being set */
  RTC_WRITE (RTC_CONTROL, save_control | RTC_SET);

  save_freq_select = RTC_READ (RTC_FREQ_SELECT); /* stop and reset prescaler */
  RTC_WRITE (RTC_FREQ_SELECT, save_freq_select | RTC_DIV_RESET2);

  rtc_minutes = RTC_READ (RTC_MINUTES);
  if (!(save_control & RTC_DM_BINARY))
    BCD_TO_BIN (rtc_minutes);

  /* Since we're only adjusting minutes and seconds, don't interfere
     with hour overflow.  This avoids messing with unknown time zones
     but requires your RTC not to be off by more than 30 minutes.  */
  if ((rtc_minutes < real_minutes
       ? real_minutes - rtc_minutes
       : rtc_minutes - real_minutes) < 30)
    {
      if (!(save_control & RTC_DM_BINARY))
	{
	  BIN_TO_BCD (real_seconds);
	  BIN_TO_BCD (real_minutes);
	}
      RTC_WRITE (RTC_SECONDS, real_seconds);
      RTC_WRITE (RTC_MINUTES, real_minutes);
    }
  else
    retval = -1;

  RTC_WRITE (RTC_FREQ_SELECT, save_freq_select);
  RTC_WRITE (RTC_CONTROL, save_control);
  return retval;
}


void ata_serial_print (const char *str)
{
}


/* ++roman:
 *
 * This function does a reset on machines that lack the ability to
 * assert the processor's _RESET signal somehow via hardware. It is
 * based on the fact that you can find the initial SP and PC values
 * after a reset at physical addresses 0 and 4. This works pretty well
 * for Atari machines, since the lowest 8 bytes of physical memory are
 * really ROM (mapped by hardware). For other 680x0 machines: don't
 * know if it works...
 *
 * To get the values at addresses 0 and 4, the MMU better is turned
 * off first. After that, we have to jump into physical address space
 * (the PC before the pmove statement points to the virtual address of
 * the code). Getting that physical address is not hard, but the code
 * becomes a bit complex since I've tried to ensure that the jump
 * statement after the pmove is in the cache already (otherwise the
 * processor can't fetch it!). For that, the code first jumps to the
 * jump statement with the (virtual) address of the pmove section in
 * an address register . The jump statement is surely in the cache
 * now. After that, that physical address of the reset code is loaded
 * into the same address register, pmove is done and the same jump
 * statements goes to the reset code. Since there are not many
 * statements between the two jumps, I hope it stays in the cache.
 *
 * The C code makes heavy use of the GCC features that you can get the
 * address of a C label. No hope to compile this with another compiler
 * than GCC!
 */

/* ++andreas: no need for complicated code, just depend on prefetch */
  
void atari_reset (void)
{
	long tc_val = 0;
	long reset_addr;

	/* On the Medusa, phys. 0x4 may contain garbage because it's no ROM */
	reset_addr = is_medusa ? 0xe00030 : *(unsigned long *) PTOV (4);

	cli();
	if (boot_info.cputype & CPU_68040) {
		unsigned long jmp_addr040 = VTOP(&&jmp_addr_label040);
		__asm__ __volatile__
			("movel    %0,d0\n\t"
			 "andl     #0xff000000,d0\n\t"
			 "orw      #0xe020,d0\n\t"   /* map 16 MB, enable, cacheable */
			 ".long    0x4e7b0004\n\t"   /* movec d0,itt0 */
			 ".long    0x4e7b0006\n\t"   /* movec d0,dtt0 */
			 "jmp   %0@\n\t"
			 : /* no outputs */
			 : "a" (jmp_addr040)
			 : "d0" );
	  jmp_addr_label040:
	    __asm__ __volatile__
			("moveq #0,d0\n\t"
			 ".long 0x4e7b0003\n\t"	/* movec d0,tc */
			 "jmp %0@"
			 : /* no outputs */
			 : "a" (reset_addr)
			 : "d0");
	}
	else
		__asm__ __volatile__
			("pmove %0@,tc\n\t"
			 "jmp %1@"
			 : /* no outputs */
			 : "a" (&tc_val), "a" (reset_addr));
}

