

/*
 * drivers/char/atari_MFPser.c: Atari MFP serial ports implementation
 *
 * Copyright 1994 Roman Hodek
 *   EMail: rnhodek@cip.informatik.uni-erlangen.de (Internet)
 *      or: Roman_Hodek@n.maus.de (MausNet, NO mail > 16 KB!)
 * Partially based on PC-Linux serial.c by Linus Torvalds and Theodore Ts'o
 * 
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file README.legal in the main directory of this archive
 * for more details.
 *
 * Modified for midi by Martin Schaller
 */


#include <linux/types.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/errno.h>
#include <linux/tty.h>
#include <linux/termios.h>
#include <linux/atarihw.h>
#include <linux/atariints.h>
#include <linux/traps.h>

#include "serial.h"
#include "atari_MFPser.h"

struct async_struct *midi_info;

void MIDI_int(void)	/* called from keyboard int handler */

{	int		ch, stat, err;
	
	stat = acia.mid_ctrl;
		
	if (stat & (ACIA_RDRF | ACIA_FE | ACIA_OVRN)) {
		ch = acia.mid_data;
		err=0;
		if (stat & ACIA_FE)
			err = TTY_FRAME;
		if (stat & ACIA_OVRN)
			err = TTY_OVERRUN;
		rs_put_char_to_queue( midi_info, ch, err );
		rs_rx_throttle_if_needed( midi_info );
	}	
	if ((! (acia.mid_ctrl & ACIA_TDRE)) || midi_info->tty->stopped || 
		midi_info->tty->hw_stopped)
		return;
	if ((ch = rs_get_char_from_queue( midi_info )) < 0) {
		/* disable tx int */
		acia.mid_ctrl = ACIA_DIV16 | ACIA_D8N1S | ACIA_RIE;
		return;
	}
	acia.mid_data = ch;
	rs_tx_wakeup_if_needed( midi_info );
}

static void MIDI_init( struct async_struct *info )

{	
	/* enable rx interrupts */
	acia.mid_ctrl = ACIA_DIV16 | ACIA_D8N1S | ACIA_RIE;
}


static void MIDI_deinit( struct async_struct *info, int leave_dtr )

{
	/* disable Rx and Tx interrupts */
	acia.mid_ctrl = ACIA_DIV16 | ACIA_D8N1S;

	/* read Rx status and data to clean up */

	(void)acia.mid_ctrl;
	(void)acia.mid_data;
}


static void MIDI_enab_tx_int( struct async_struct *info, int enab_flag )

{
	if (enab_flag)
		acia.mid_ctrl = ACIA_DIV16 | ACIA_D8N1S | ACIA_RIE | 
				ACIA_RLTIE;
	else
		acia.mid_ctrl = ACIA_DIV16 | ACIA_D8N1S | ACIA_RIE;
}


static int MIDI_check_custom_divisor( struct async_struct *info,
					int divisor )

{	
	return (-1);
}

static void MIDI_change_speed( struct async_struct *info )

{
	/* no-op */
}
	
	
static void MIDI_restart( struct async_struct *info )

{
	/* not needed for midi */
}


static void MIDI_throttle( struct async_struct *info, int status )

{
	/* no-op */
}


static void MIDI_set_break( struct async_struct *info, int break_flag )

{
	/* no-op */
}

static unsigned int MIDI_get_modem_info( struct async_struct *info )

{
	return( TIOCM_RTS | TIOCM_DTR | TIOCM_CAR | TIOCM_CTS | TIOCM_DSR );
}


static void MIDI_get_serial_info( struct async_struct *info,
								   struct serial_struct *retinfo )

{
	retinfo->baud_base = 31250;
	retinfo->custom_divisor = 1;
}

static int MIDI_set_modem_info( struct async_struct *info,
				int new_dtr, int new_rts )

{
	/* no-op */

	/* Is it right to return an error or should the attempt to change
	 * DTR or RTS be silently ignored?
	 */
	return( -EINVAL );
}

/* SERIALSWITCH structures for MIDI port
 */

static SERIALSWITCH MIDI_switch = {
	MIDI_init, MIDI_deinit, MIDI_enab_tx_int,
	MIDI_check_custom_divisor, MIDI_change_speed,
	MIDI_restart, MIDI_throttle, MIDI_set_break,
	MIDI_get_serial_info, MIDI_get_modem_info,
	MIDI_set_modem_info, NULL
};

void atari_init_MIDIser(struct async_struct *info)
{	
	midi_info=info;
	info->type = MIDI;
	info->base = &(acia.mid_ctrl);
	info->sw = &MIDI_switch;
	acia.mid_ctrl = ACIA_DIV16 | ACIA_D8N1S;
}


