/*
 * Atari Mouse Driver for Linux
 * by Robert de Vries (robert@and.nl) 19Jul93
 *
 * 16 Nov 1994 Andreas Schwab
 * Compatibility with busmouse
 * Support for three button mouse (shamelessly stolen from MiNT)
 * third button wired to one of the joystick directions on joystick 1
 */

#include <linux/sched.h>
#include <linux/errno.h>
#include <linux/atarikb.h>
#include <linux/mouse.h>
#include <linux/atari_mouse.h>
#include <asm/segment.h>

static struct mouse_status mouse;
int atari_mouse_buttons;

void atari_mouse_interrupt(char *buf)
{
    int buttons;

/*    ikbd_mouse_disable(); */

    buttons = ((buf[0] & 1 ? 1 : 0)
	       | (buf[0] & 2 ? 4 : 0)
	       | (atari_mouse_buttons & 2));
    atari_mouse_buttons = buttons;
    mouse.buttons = ~buttons & 7;
    mouse.dx += buf[1];
    mouse.dy -= buf[2];
    mouse.ready = 1;
    wake_up_interruptible(&mouse.wait);

/*    ikbd_mouse_rel_pos(); */
}

static void release_mouse(struct inode *inode, struct file *file)
{
    ikbd_mouse_disable();

    mouse.active = 0;
    mouse.ready = 1;
}

static int open_mouse(struct inode *inode, struct file *file)
{
    int minor = MINOR(inode->i_rdev);

    if (minor != ATARIMOUSE_MINOR)
	return -ENODEV;
    if (mouse.active)
	return -EBUSY;
    mouse.active = 1;
    mouse.ready = 0;
    mouse.dx = mouse.dy = 0;
    atari_mouse_buttons = 0;
    ikbd_mouse_y0_top ();
    ikbd_mouse_thresh (1, 1);
    ikbd_mouse_rel_pos();
    return 0;
}

static int write_mouse(struct inode *inode, struct file *file, char *buffer, int count)
{
    return -EINVAL;
}

static int read_mouse(struct inode *inode, struct file *file, char *buffer, int count)
{
    int dx, dy, buttons;

    if (count < 3)
	return -EINVAL;
    if (!mouse.ready)
	return -EAGAIN;
    /* ikbd_mouse_disable */
    dx = mouse.dx;
    dy = mouse.dy;
    buttons = mouse.buttons;
    if (dx > 127)
      dx = 127;
    else if (dx < -128)
      dx = -128;
    if (dy > 127)
      dy = 127;
    else if (dy < -128)
      dy = -128;
    mouse.dx -= dx;
    mouse.dy -= dy;
    mouse.ready = 0;
    /* ikbd_mouse_rel_pos(); */
    put_fs_byte(buttons | 0x80, buffer++);
    put_fs_byte((char) dx, buffer++);
    put_fs_byte((char) dy, buffer++);
    return 3;
}

static int mouse_select(struct inode *inode, struct file *file, int sel_type, select_table *wait)
{
    if (sel_type != SEL_IN)
	return 0;
    if (mouse.ready)
	return 1;
    select_wait(&mouse.wait, wait);
    return 0;
}

struct file_operations atari_mouse_fops = {
    NULL,		/* mouse_seek */
    read_mouse,
    write_mouse,
    NULL,		/* mouse_readdir */
    mouse_select,
    NULL,		/* mouse_ioctl */
    NULL,		/* mouse_mmap */
    open_mouse,
    release_mouse
};

unsigned long atari_mouse_init(unsigned long kmem_start)
{
    mouse.active = 0;
    mouse.ready = 0;
    mouse.wait = NULL;

    printk("Atari mouse installed.\n");
    return kmem_start;
}
