/*
 * WD33C93 definitions
 */

#ifndef WD33C93_H
#define WD33C93_H

/* wd register names */
#define WD_OWN_ID		0x00
#define WD_CONTROL		0x01
#define WD_TIMEOUT_PERIOD	0x02
#define WD_CDB_1		0x03
#define WD_CDB_2		0x04
#define WD_CDB_3		0x05
#define WD_CDB_4		0x06
#define WD_CDB_5		0x07
#define WD_CDB_6		0x08
#define WD_CDB_7		0x09
#define WD_CDB_8		0x0a
#define WD_CDB_9		0x0b
#define WD_CDB_10		0x0c
#define WD_CDB_11		0x0d
#define WD_CDB_12		0x0e
#define WD_TARGET_LUN		0x0f
#define WD_COMMAND_PHASE	0x10
#define WD_SYNCHRONOUS_TRANSFER 0x11
#define WD_TRANSFER_COUNT_MSB	0x12
#define WD_TRANSFER_COUNT	0x13
#define WD_TRANSFER_COUNT_LSB	0x14
#define WD_DESTINATION_ID	0x15
#define WD_SOURCE_ID		0x16
#define WD_SCSI_STATUS		0x17
#define WD_COMMAND		0x18
#define WD_DATA 		0x19
#define WD_AUXILIARY_STATUS	0x1f

/* WD commands */
#define WD_CMD_RESET		0x00
#define WD_CMD_ABORT		0x01
#define WD_CMD_ASSERT_ATN	0x02
#define WD_CMD_NEGATE_ACK	0x03
#define WD_CMD_DISCONNECT	0x04
#define WD_CMD_RESELECT 	0x05
#define WD_CMD_SEL_ATN		0x06
#define WD_CMD_SEL		0x07
#define WD_CMD_SEL_ATN_XFER	0x08
#define WD_CMD_SEL_XFER 	0x09
#define WD_CMD_RESEL_RECEIVE	0x0a
#define WD_CMD_RESEL_SEND	0x0b
#define WD_CMD_WAIT_SEL_RECEIVE 0x0c
#define WD_CMD_TRANS_ADDR	0x18
#define WD_CMD_TRANS_INFO	0x20
#define WD_CMD_TRANSFER_PAD	0x21
#define WD_CMD_SBT_MODE 	0x80

/* ASR register */
#define ASR_INT			(0x80)
#define ASR_LCI			(0x40)
#define ASR_BSY			(0x20)
#define ASR_CIP			(0x10)
#define ASR_PE			(0x02)
#define ASR_DBR			(0x01)

/* Phases */
#define PHS_DATA_OUT		0x00
#define PHS_DATA_IN		0x01
#define PHS_COMMAND		0x02
#define PHS_STATUS		0x03
#define PHS_MESS_OUT		0x06
#define PHS_MESS_IN		0x07

/* Command Status Register definitions */
  /* reset state interrupts */
#define CSR_RESET		0x00
#define CSR_RESET_AF		0x01
  /* successful completion interrupts */
#define CSR_RESELECT		0x10
#define CSR_SELECT		0x11
#define CSR_SEL_XFER_DONE	0x16
#define CSR_XFER_DONE		0x18

  /* paused or aborted interrupts */
#define CSR_MSGIN		0x20
#define CSR_SDP			0x21
#define CSR_SEL_ABORT		0x22
#define CSR_RESEL_ABORT		0x25
#define CSR_RESEL_ABORT_AM	0x27
#define CSR_ABORT		0x28

  /* terminated interrupts */
#define CSR_INVALID		0x40
#define CSR_UNEXP_DISC		0x41
#define CSR_TIMEOUT		0x42
#define CSR_PARITY		0x43
#define CSR_PARITY_ATN		0x44
#define CSR_BAD_STATUS		0x45
#define CSR_UNEXP		0x48

  /* service required interrupts */
#define CSR_RESEL		0x80
#define CSR_RESEL_AM		0x81
#define CSR_DISC		0x85
#define CSR_SRV_REQ		0x88

  /* clock frequency bits */
#define WD33C93_FS_8_10         (0x00)
#define WD33C93_FS_12_15	(0x01)
#define WD33C93_FS_16_20	(0x02)

/* extended message codes */
#define SYNC_DATA_XFER_REQ      (0x01)
/*
 * Debug definitions
 */
#define NDEBUG_ARBITRATION	0x1
#define NDEBUG_AUTOSENSE	0x2
#define NDEBUG_DMA		0x4
#define NDEBUG_HANDSHAKE	0x8
#define NDEBUG_INFORMATION	0x10
#define NDEBUG_INIT		0x20
#define NDEBUG_INTR		0x40
#define NDEBUG_LINKED		0x80
#define NDEBUG_MAIN		0x100
#define NDEBUG_NO_DATAOUT	0x200
#define NDEBUG_NO_WRITE		0x400
#define NDEBUG_PIO		0x800
#define NDEBUG_PSEUDO_DMA	0x1000
#define NDEBUG_QUEUES		0x2000
#define NDEBUG_RESELECTION	0x4000
#define NDEBUG_SELECTION	0x8000
#define NDEBUG_USLEEP		0x10000

#define NDEBUG 0xFFFFFFFF

typedef struct {
	volatile unsigned char   SASR;
	char                     pad;
	volatile unsigned char   SCMD;
} wd33c93_regs;

typedef int (*dma_setup_t) (Scsi_Cmnd *SCpnt, int dir_in);
typedef void (*dma_stop_t) (struct Scsi_Host *instance, Scsi_Cmnd *SCpnt,
			    int status);

struct WD33C93_hostdata {
    struct Scsi_Host *next;
    wd33c93_regs     *regp;
    int              clock_freq;
    int              pio;			/* programmed I/O */
    int		     dma_dir;			/* data transfer dir. */
    dma_setup_t      dma_setup;
    dma_stop_t       dma_stop;
    unsigned char    *dma_bounce_buffer;
    unsigned int     dma_bounce_len;
    volatile unsigned char busy[8];		/* index = target, bit = lun */
    volatile int running;
    volatile Scsi_Cmnd *connected;		/* currently connected command */
    volatile Scsi_Cmnd *issue_queue;		/* waiting to be issued */
    volatile Scsi_Cmnd *disconnected_queue;	/* waiting for reconnect */
    unsigned char    first;                     /* bit = target */
};

void wd33c93_init (struct Scsi_Host *instance, wd33c93_regs *regs,
		   dma_setup_t setup, dma_stop_t stop, int clock_freq);
int wd33c93_abort (Scsi_Cmnd *cmd, int code);
int wd33c93_reset (Scsi_Cmnd *);
int wd33c93_queuecommand (Scsi_Cmnd *cmd, void (*done)(Scsi_Cmnd *));
void wd33c93_intr (struct Scsi_Host *instance);

#endif /* WD33C93_H */
