/*
** atariints.h -- Atari Linux interrupt handling structs and prototypes
**
** Copyright 1994 by Bjrn Brauel
**
** 5/2/94 Roman Hodek:
**   TT interupt definitions added.
**
** This file is subject to the terms and conditions of the GNU General Public
** License.  See the file README.legal in the main directory of this archive
** for more details.
**
*/

#ifndef _LINUX_ATARIINTS_H_
#define _LINUX_ATARIINTS_H_

/*
** Atari Interrupt sources.
**
*/

#define NUM_ATARI_SOURCES   (48)
#define STMFP_SOURCE_BASE   0
#define TTMFP_SOURCE_BASE   16
#define SCC_SOURCE_BASE	    32

#define IRQ_MFP_BUSY      (IRQ_MACHSPEC | 0)
#define IRQ_MFP_DCD       (IRQ_MACHSPEC | 1)
#define IRQ_MFP_CTS  	  (IRQ_MACHSPEC | 2)
#define IRQ_MFP_GPU 	  (IRQ_MACHSPEC | 3)
#define IRQ_MFP_TIMD      (IRQ_MACHSPEC | 4)
#define IRQ_MFP_TIMC	  (IRQ_MACHSPEC | 5)
#define IRQ_MFP_ACIA	  (IRQ_MACHSPEC | 6)
#define IRQ_MFP_FDC       (IRQ_MACHSPEC | 7)
#define IRQ_MFP_TIMB      (IRQ_MACHSPEC | 8)
#define IRQ_MFP_SERERR    (IRQ_MACHSPEC | 9)
#define IRQ_MFP_SEREMPT   (IRQ_MACHSPEC | 10)
#define IRQ_MFP_RECERR    (IRQ_MACHSPEC | 11)
#define IRQ_MFP_RECFULL   (IRQ_MACHSPEC | 12)
#define IRQ_MFP_TIMA      (IRQ_MACHSPEC | 13)
#define IRQ_MFP_RI        (IRQ_MACHSPEC | 14)
#define IRQ_MFP_MMD       (IRQ_MACHSPEC | 15)

#define IRQ_FLOPPY	    IRQ_MFP_FDC

#define IRQ_TT_MFP_IO0       (IRQ_MACHSPEC | 16)
#define IRQ_TT_MFP_IO1       (IRQ_MACHSPEC | 17)
#define IRQ_TT_MFP_SCC	     (IRQ_MACHSPEC | 18)
#define IRQ_TT_MFP_RI 	     (IRQ_MACHSPEC | 19)
#define IRQ_TT_MFP_TIMD      (IRQ_MACHSPEC | 20)
#define IRQ_TT_MFP_TIMC	     (IRQ_MACHSPEC | 21)
#define IRQ_TT_MFP_DRVRDY    (IRQ_MACHSPEC | 22)
#define IRQ_TT_MFP_SCSIDMA   (IRQ_MACHSPEC | 23)
#define IRQ_TT_MFP_TIMB      (IRQ_MACHSPEC | 24)
#define IRQ_TT_MFP_SERERR    (IRQ_MACHSPEC | 25)
#define IRQ_TT_MFP_SEREMPT   (IRQ_MACHSPEC | 26)
#define IRQ_TT_MFP_RECERR    (IRQ_MACHSPEC | 27)
#define IRQ_TT_MFP_RECFULL   (IRQ_MACHSPEC | 28)
#define IRQ_TT_MFP_TIMA      (IRQ_MACHSPEC | 29)
#define IRQ_TT_MFP_RTC       (IRQ_MACHSPEC | 30)
#define IRQ_TT_MFP_SCSI      (IRQ_MACHSPEC | 31)

#define	IRQ_SCCB_TX	     (IRQ_MACHSPEC | 32)
#define	IRQ_SCCB_STAT	     (IRQ_MACHSPEC | 34)
#define	IRQ_SCCB_RX	     (IRQ_MACHSPEC | 36)
#define	IRQ_SCCB_SPCOND	     (IRQ_MACHSPEC | 38)
#define	IRQ_SCCA_TX	     (IRQ_MACHSPEC | 40)
#define	IRQ_SCCA_STAT	     (IRQ_MACHSPEC | 42)
#define	IRQ_SCCA_RX	     (IRQ_MACHSPEC | 44)
#define	IRQ_SCCA_SPCOND	     (IRQ_MACHSPEC | 46)

/* INTREQR masks */
#define IRQ1_MASK   0x0007	/* INTREQR mask for IRQ 1 */
#define IRQ2_MASK   0x0008	/* INTREQR mask for IRQ 2 */
#define IRQ3_MASK   0x0070	/* INTREQR mask for IRQ 3 */
#define IRQ4_MASK   0x0780	/* INTREQR mask for IRQ 4 */
#define IRQ5_MASK   0x1800	/* INTREQR mask for IRQ 5 */
#define IRQ6_MASK   0x2000	/* INTREQR mask for IRQ 6 */
#define IRQ7_MASK   0x4000	/* INTREQR mask for IRQ 7 */

#define IF_SETCLR   0x8000      /* set/clr bit */
#define IF_INTEN  0x4000	
#define IF_I1     0x2000	
#define IF_I2     0x1000	
#define IF_I3	    0x0800	
#define IF_I4     0x0400	
#define IF_I5     0x0200	
#define IF_I6     0x0100	
#define IF_I7     0x0080	
#define IF_I8     0x0040	
#define IF_I9     0x0020	
#define IF_I10    0x0010	
#define IF_I11    0x0008	
#define IF_I12    0x0004	
#define IF_I13    0x0002	
#define IF_I14    0x0001	

#define INT_CLK   24576	    /* CLK while int_clk =2.456MHz and divide = 100 */
#define INT_TICKS 246	    /* to make sched_time = 99.902... HZ */

#endif /* linux/atariints.h */
