/*
** bootinfo.h -- Defintion of the Linux/68K boot information structure
**
** Copyright 1992 by Greg Harp
**
** This file is subject to the terms and conditions of the GNU General Public
** License.  See the file README.legal in the main directory of this archive
** for more details.
**
** Created 09/29/92 by Greg Harp
**
** 5/2/94 Roman Hodek:
**   Added bi_atari part of the machine dependant nion bi_un; for now it
**	 contains just a model field to distinguish between TT and Falcon.
*/

#ifndef BOOTINFO_H
#define BOOTINFO_H

#include <linux/amigatypes.h>

/*
 * Amiga specific part of bootinfo structure.
 */

#define NUM_AUTO    16

#define NTSC_ECLOCK 715909
#define PAL_ECLOCK  709379

struct bi_Amiga {
    int
        model;				/* Amiga Model (3000?) */
    int
	num_autocon;			/* # of autoconfig devices found */

    struct ConfigDev
	autocon[NUM_AUTO];		/* up to 16 autoconfig devices */

    unsigned long
	chip_size;			/* size of chip memory (bytes) */

    unsigned char
	vblank; 			/* VBLANK frequency */

    unsigned char
	psfreq; 			/* power supply frequency */

    unsigned long
	eclock; 			/* EClock frequency */

    unsigned long
	chipset;			/* native chipset present */
};


/* Atari specific part of bootinfo */

struct bi_Atari {
	unsigned long 	hw_present;
};

/*
 * Define several Hardware-Chips for indication so that for the ATARI we do
 * no longer decide whether it is a Falcon or other machine . It's just
 * important what hardware the machine uses
 */

/* video-hardware */
#define ATARIHW_STND_SHIFTER	(0x000001)	/* ST-Shifter - no base low ! */
#define ATARIHW_EXTD_SHIFTER	(0x000002)	/* STe-Shifter - 24 bit adress */
#define ATARIHW_TT_SHIFTER	(0x000004)	/* TT-Shifter */
#define ATARIHW_VIDEL_SHIFTER	(0x000008)	/* Falcon-Shifter */
#define ATARIHW_EXTERN_VIDEO	(0x000010)	/* External Video e.g ET4000 */
/* sound-hardware */
#define ATARIHW_YM_2149		(0x000020)	/* Yamaha YM 2149 */
#define ATARIHW_PCM_8BIT	(0x000040)	/* PCM-Sound in STe-ATARI */
#define ATARIHW_CODEC_16BIT	(0x000080)	/* CODEC-Sound-Processor in Falcon */
/* IO-hardware */
#define ATARIHW_TT_SCSI		(0x000100)	/* Directly mapped TT-SCSI NCR5380 */
#define ATARIHW_STND_ACSI	(0x000200)	/* Standard ACSI like in STs */
#define ATARIHW_STND_SCSI	(0x000400)	/* Standard SCSI for Falcon */
#define ATARIHW_IDE		(0x000800)	/* IDE Interface */
#define ATARIHW_WD_HD		(0x001000)	/* HD-Floppy support */
#define ATARIHW_ST_MFP		(0x002000)	/* The ST-Mfp (no ATARI without it .. but who cares) */
#define ATARIHW_TT_MFP		(0x004000)	/* 2nd MFP only in TT and compatibles */
#define ATARIHW_SCC		(0x008000)	/* Serial Comminucations Controller */
#define ATARIHW_ANALOG_JOY	(0x010000)	/* Paddle Interface for Ste and Falcon */
/* DMA-hardware */
#define ATARIHW_STND_DMA	(0x020000)	/* 24 Bit limited DMA */
#define ATARIHW_EXTD_DMA	(0x040000)	/* 32 Bit unlimited DMA */
#define ATARIHW_SCSI_DMA	(0x080000)	/* DMA for the NCR 5380 */
#define ATARIHW_SCC_DMA		(0x100000)	/* DMA for the SCC */
/* util-hardware */
#define ATARIHW_SCU		(0x200000)	/* System Control Unit in TT and MSTe */
#define ATARIHW_BLITTER		(0x400000)	/* Blitter */
#define ATARIHW_VME		(0x800000)	/* The VME-Bus */
#define ATARIHW_MCW		(0x1000000)	/* Microwire Interface */
#define ATARIHW_TT_CLK		(0x2000000)	/* TT-Compatible Clock-Chip */
#define ATARIHW_MSTE_CLK	(0x4000000)	/* Mega ST(E) Clock-Chip */

/*
 * CPU and FPU types
 */
#define CPU_68020    (1)
#define CPU_68030    (2)
#define CPU_68040    (4)
#define CPU_MASK     (31)
#define FPU_68881    (32)
#define FPU_68882    (64)

struct mem_info {
    unsigned long addr;		/* physical address of memory chunk */
    unsigned long size;		/* length of memory chunk (in bytes) */
};

#define NUM_MEMINFO  4

#define MACH_AMIGA   1
#define MACH_ATARI   2
#define MACH_MAC     3

#define CL_SIZE      (80)

struct bootinfo {
    unsigned long
	machtype;			/* machine type */

    unsigned long
	cputype;			/* system CPU & FPU */

    struct mem_info
	memory[NUM_MEMINFO];		/* memory description */

    int
	num_memory;			/* # of memory blocks found */

    unsigned long
	ramdisk_size;			/* ramdisk size in 1024 byte blocks */

    unsigned long
	ramdisk_addr;			/* address of the ram disk in mem */

    char
        command_line[CL_SIZE];		/* kernel command line parameters */

    union {
	struct bi_Amiga bi_ami; 	/* Amiga specific information */
	struct bi_Atari bi_ata; 	/* Atari specific information */
    } bi_un;
};
#define bi_amiga bi_un.bi_ami
#define bi_atari bi_un.bi_ata
#define bi_mac	 bi_un.bi_mac

extern struct bootinfo
    boot_info;

#endif /* BOOTINFO_H */
