#ifndef _LINUX_DELAY_H
#define _LINUX_DELAY_H

/*
 * Copyright (C) 1993 Linus Torvalds
 *
 * Delay routines, using a pre-computed "loops_per_second" value.
 */

extern unsigned long loops_per_sec;

#if defined(__i386__)
extern __inline__ void __delay(int loops)
{
	__asm__(".align 2,0x90\n1:\tdecl %0\n\tjns 1b": :"a" (loops):"ax");
}
#elif defined(__mc68000__)
extern __inline__ void __delay(int loops)
{
	__asm__("\n\tmovel %0,d0\n1:\tsubql #1,d0\n\tbpls 1b\n"
		: /* no outputs */
		: "g" (loops)
		: "d0");
}
#endif

/*
 * division by multiplication: you don't have to worry about
 * loss of precision.
 *
 * Use only for very small delays ( < 1 msec).  Should probably use a
 * lookup table, really, as the multiplications take much too long with
 * short delays.  This is a "reasonable" implementation, though (and the
 * first constant multiplications gets optimized away if the delay is
 * a constant)  
 */
extern __inline__ void udelay(unsigned long usecs)
{
	usecs *= 0x000010c6;		/* 2**32 / 1000000 */
#if defined(__i386__)
	__asm__("mull %0"
		:"=d" (usecs)
		:"a" (usecs),"0" (loops_per_sec)
		:"ax");
#elif defined(__mc68000__)
	asm ("mulul %1,%0:%2"
	     : "=d" (usecs)
	     : "d" (usecs),
	       "d" (loops_per_sec));
#endif
	__delay(usecs);
}

#endif
