#ifndef _LINUX_FB_H
#define _LINUX_FB_H

/* Definitions of frame buffers						*/

#ifdef __KERNEL__
#include <linux/fs.h>
#endif

/* ioctls
   0x46 is 'F'								*/
#define FBIOGET_VSCREENINFO 	0x4600
#define FBIOPUT_VSCREENINFO 	0x4601
#define FBIOGET_FSCREENINFO 	0x4602
#define FBIOGETCMAP		0x4604
#define FBIOPUTCMAP		0x4605

#define FB_TYPE_PACKED_PIXELS		0	/* Packed Pixels	*/
#define FB_TYPE_PLANES			1	/* Non interleaved planes */
#define FB_TYPE_INTERLEAVED_PLANES	2	/* Interleaved planes	*/

#define FB_VISUAL_MONO01		0	/* Monochr. 1=Black 0=White */
#define FB_VISUAL_MONO10		1	/* Monochr. 1=White 0=Black */
#define FB_VISUAL_TRUECOLOR		2	/* True color	*/
#define FB_VISUAL_PSEUDOCOLOR		3	/* Pseudo color (like atari) */
#define FB_VISUAL_DIRECTCOLOR		4	/* Direct color */
#define FB_VISUAL_STATIC_PSEUDOCOLOR	5	/* Pseudo color readonly */
#define FB_VISUAL_STATIC_DIRECTCOLOR	6	/* Direct color readonly */

struct fb_fix_screeninfo {
	char id[16];			/* identification string eg "TT Builtin" */
	unsigned long smem_start;	/* Start of frame buffer mem */
	unsigned long smem_len;		/* Length of frame buffer mem */	
	int type;			/* see FB_TYPE_* 		*/
	int type_aux;			/* Interleave for interleaved Planes */
	int visual;			/* see FB_VISUAL_*  		*/ 
	int reserved[7];		/* Reserved for future compatibility */
};

struct fb_bitfield {
	int offset;			/* beginning of bitfield	*/
	int length;			/* length of bitfield		*/
	int msb_right;			/* != 0 : Most significant bit is */ 
					/* right */ 
};

#define FB_ACTIVATE_NOW		0	/* set values immediately (or vbl)*/
#define FB_ACTIVATE_NXTOPEN	1	/* activate on next open	*/
#define FB_ACTIVATE_TEST	2	/* don't set, round up impossible */
#define FB_ACTIVATE_MASK       15
					/* values			*/
#define FB_ACTIVATE_VBL	       16	/* activate values on next vbl  */
#define FB_CHANGE_CMAP_VBL     32	/* change colormap on vbl	*/

#define FB_ACCEL_NONE		0	/* no hardware accelerator	*/
#define FB_ACCEL_ATARIBLITT	1	/* Atari Blitter		*/

#define FB_SYNC_HOR_HIGH_ACT	1	/* horizontal sync high active	*/
#define FB_SYNC_VERT_HIGH_ACT	2	/* vertical sync high active	*/
#define FB_SYNC_EXT		4	/* external sync		*/

#define FB_VMODE_NONINTERLACED  0	/* non interlaced */
#define FB_VMODE_INTERLACED 	1	/* interlaced	*/
#define FB_VMODE_DOUBLE		2	/* double scan */

struct fb_var_screeninfo {
	int xres;			/* visible resolution		*/
	int yres;
	int xres_virtual;		/* virtual resolution		*/
	int yres_virtual;
	int xoffset;			/* offset from virtual to visible */
	int yoffset;			/* resolution			*/

	int bits_per_pixel;		/* guess what 			*/
	int grayscale;			/* != 0 Graylevels instead of colors */

	struct fb_bitfield red;		/* bitfield in fb mem if true color, */
	struct fb_bitfield green;	/* else only length is significant */
	struct fb_bitfield blue;
	struct fb_bitfield transp;	/* transparency			*/	

	int nonstd;			/* != 0 Non standard pixel format */

	int activate;			/* see FB_ACTIVATE_* 		*/

	int height;			/* height of picture in mm    */
	int width;			/* width of picture in mm     */

	int accel;			/* see FB_ACCEL_*		*/

	/* Timing All values in pixclocks, except pixclock (of course) 	*/
	unsigned long pixclock;		/* pixel clock in ps (pico seconds) */
	unsigned long left_margin;	/* time from sync to picture	*/
	unsigned long right_margin;	/* time from picture to sync	*/
	unsigned long upper_margin;	/* time from sync to picture	*/
	unsigned long lower_margin;
	unsigned long hsync_len;	/* length of horizontal sync	*/
	unsigned long vsync_len;	/* length of vertical sync	*/
	int sync;			/* see FB_SYNC_*		*/
	int vmode;			/* see FB_VMODE_*		*/
	int reserved[6];		/* Reserved for future compatibility */
};

struct fb_cmap {
	int start;			/* First entry	*/
	int len;			/* Number of entrys */
	unsigned short *red;		/* Red values	*/
	unsigned short *green;
	unsigned short *blue;
	unsigned short *transp;		/* transparency, can be NULL */
};

#ifdef __KERNEL__

struct fb_ops {
	/* get non setable parameters	*/
	int (*fb_get_fix) (struct fb_fix_screeninfo *, int); 
	/* get setable parameters	*/
	int (*fb_get_var) (struct fb_var_screeninfo *, int);		
	/* set setable parameters	*/
	int (*fb_set_var) (struct fb_var_screeninfo *, int);		
	/* get colormap			*/
	int (*fb_get_cmap) (struct fb_cmap *, int, int);
	/* set colormap			*/
	int (*fb_set_cmap) (struct fb_cmap *, int, int);
	/* perform fb specific ioctl	*/
	int (*fb_ioctl)(struct inode *, struct file *, unsigned int,
			unsigned long, int);
};

int register_framebuffer(char *, int *, struct fb_ops *, int, 
			 struct fb_var_screeninfo *);
int unregister_framebuffer(int);
#endif /* __KERNEL__ */

#endif /* _LINUX_FB_H */
